import { Database, } from "@orbitdb/core";
const type = "ordered-keyvalue";
const OrderedKeyValue = () => async ({ ipfs, identity, address, name, access, directory, meta, headsStorage, entryStorage, indexStorage, referencesCount, syncAutomatically, onUpdate, }) => {
    const database = await Database({
        ipfs,
        identity,
        address,
        name,
        access,
        directory,
        meta,
        headsStorage,
        entryStorage,
        indexStorage,
        referencesCount,
        syncAutomatically,
        onUpdate,
    });
    const { addOperation, log } = database;
    const put = async (key, value, position) => {
        const entryValue = { value };
        if (position !== undefined) {
            entryValue.position = position;
        }
        return addOperation({ op: "PUT", key, value: entryValue });
    };
    const move = async (key, position) => {
        return addOperation({ op: "MOVE", key, value: position });
    };
    const del = async (key) => {
        return addOperation({ op: "DEL", key, value: null });
    };
    const get = async (key) => {
        for await (const entry of log.traverse()) {
            const { op, key: k, value } = entry.payload;
            if (op === "PUT" && k === key) {
                return value;
            }
            else if (op === "DEL" && k === key) {
                return undefined;
            }
        }
        return undefined;
    };
    const iterator = async function* ({ amount, } = {}) {
        const keys = {};
        const positions = {};
        let count = 0;
        for await (const entry of log.traverse()) {
            const { op, key, value } = entry.payload;
            if (!key)
                return;
            if (op === "PUT" && !keys[key]) {
                keys[key] = true;
                const putValue = value;
                const hash = entry.hash;
                const position = positions[key] !== undefined
                    ? positions[key]
                    : putValue.position !== undefined
                        ? putValue.position
                        : 0;
                positions[key] = position;
                count++;
                yield { key, value: putValue.value, position, hash };
            }
            else if (op === "MOVE" && !keys[key]) {
                positions[key] = value;
            }
            else if (op === "DEL" && !keys[key]) {
                keys[key] = true;
            }
            if (amount !== undefined && count >= amount) {
                break;
            }
        }
    };
    const all = async () => {
        const values = [];
        for await (const entry of iterator()) {
            values.unshift(entry);
        }
        return values
            .sort((a, b) => a.position > b.position ? 1 : a.position === b.position ? 0 : -1)
            .map((x) => ({
            key: x.key,
            value: x.value,
            hash: x.hash,
        }));
    };
    return {
        ...database,
        type,
        put,
        set: put,
        del,
        move,
        get,
        iterator,
        all,
    };
};
OrderedKeyValue.type = type;
export default OrderedKeyValue;
//# sourceMappingURL=ordered-keyvalue.js.map